#include <XSstreams.h>
#include <xsTypes.h>
#include <XSContainer.h>
#include <XSFit/Fit/Fit.h>
#include <XSFit/MCMC/MarginGrid.h>
#include <XSFit/MCMC/ChainManager.h>
#include <XSUser/Global/Global.h>
#include <XSUser/Global/XSGlobal.h>
#include <XSUser/Handler/HandlerUtils.h>

int
XSGlobal::xsMargin(ClientData cdata,Tcl_Interp* tclInterp,int objc, Tcl_Obj* CONST  objv[])
{
   // Margin command is very similar to steppar, though somewhat
   // simpler.  It doesn't use current/best setting, nor does
   // it require fits to be performed.
   Grid::SpecContainer newSettings;
   static Grid::ParameterSpec* prevSettings = 0;
   ChainManager* chainManager = XSContainer::fit->chainManager();   
   
   string cmd = string(static_cast<char*>(cdata));
   bool usePreviousSettings(objc == 1);
   if (usePreviousSettings && !prevSettings)
   {
      tcout << "No parameters yet specified: summary of command:\n";
      printDocs(cmd.c_str(),"?");
      return globalData->autoSave(TCL_OK);          
   }
   static MarginGrid* grid = 0;
   if (!grid)
   {
      grid = new MarginGrid(chainManager);
      chainManager->marginGrid(grid);
   }
   
   try
   {
      if (!usePreviousSettings)
      {
         string arg(Tcl_GetString(objv[1]));
         if (arg[0] == '?')
         {
	     printDocs(cmd.c_str(),"?");
	     return globalData->autoSave(TCL_OK);              
         }
         else
         {
            bool isBest = false; // dummy variable in this context
            // The following can allocate new memory into 
            // newSettings array, and prevSettings will have
            // same pointer value as the last entry in array.
            HandlerUtils::commonStepParsing(objc, objv, prevSettings,
                newSettings, isBest, false);
            if (!newSettings.size())
            {
               return globalData->autoSave(TCL_ERROR);
            }
            // commonStepParsing has set prevSettings to most
            // recent entry in newSettings array, which will now
            // be owned by grid.  
            grid->reinitialize(newSettings);
            grid->doGrid();
         }
      }
      grid->report();
   }
   catch (YellowAlert&)
   {
      // If something went wrong, remove grid.  Otherwise a grid
      // in an undefined state could be used later for "plot margin".
      delete grid;
      grid = 0;
      // prevSettings will point to invalid memory once grid is
      // deleted (see above), so...
      prevSettings = 0;
      chainManager->marginGrid(0); 
   }
   
   return globalData->autoSave(TCL_OK);
}
